"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const LiquidCompletionParams_1 = require("./LiquidCompletionParams");
const documents_1 = require("../../documents");
(0, vitest_1.describe)('Module: LiquidCompletionParams', async () => {
    (0, vitest_1.describe)('createLiquidCompletionParams', async () => {
        (0, vitest_1.describe)('parsed LiquidHTML template', async () => {
            (0, vitest_1.it)('returns an undefined completionContext when the template is unsalvageable', () => {
                const context = '{{ }}%}%{%}%$}}{#$%}{% ren█ %}';
                const { completionContext } = createLiquidParamsFromContext(context);
                (0, vitest_1.expect)(completionContext).not.to.exist;
            });
            (0, vitest_1.describe)('completionContext.partialAst', async () => {
                (0, vitest_1.it)('returns an ast of the file up to the cursor position', async () => {
                    const context = '{{ "hey" }}\n\n{{ product.id }}{% ren█ %}{% echo "not in the AST" %}';
                    const { completionContext } = createLiquidParamsFromContext(context);
                    (0, vitest_1.expect)(completionContext).to.exist;
                    const { partialAst } = completionContext;
                    expectPath(partialAst, 'type').to.eql('Document');
                    expectPath(partialAst, 'name').to.eql('#document');
                    expectPath(partialAst, 'children.0.type').to.eql('LiquidVariableOutput');
                    expectPath(partialAst, 'children.0.markup.type').to.eql('LiquidVariable');
                    expectPath(partialAst, 'children.0.markup.expression.type').to.eql('String');
                    expectPath(partialAst, 'children.0.markup.expression.value').to.eql('hey');
                    expectPath(partialAst, 'children.0.markup.rawSource').to.eql('"hey"');
                    expectPath(partialAst, 'children.1.type').to.eql('LiquidVariableOutput');
                    expectPath(partialAst, 'children.1.markup.type').to.eql('LiquidVariable');
                    expectPath(partialAst, 'children.1.markup.expression.type').to.eql('VariableLookup');
                    expectPath(partialAst, 'children.1.markup.expression.name').to.eql('product');
                    expectPath(partialAst, 'children.1.markup.expression.lookups.0.type').to.eql('String');
                    expectPath(partialAst, 'children.1.markup.expression.lookups.0.value').to.eql('id');
                    expectPath(partialAst, 'children.1.markup.rawSource').to.eql('product.id');
                    expectPath(partialAst, 'children.2.name').to.eql('ren');
                    expectPath(partialAst, 'children.2.markup').to.eql('');
                    expectPath(partialAst, 'children.2.type').to.eql('LiquidTag');
                    expectPath(partialAst, 'children.3').not.to.exist;
                });
            });
            (0, vitest_1.describe)('completionContext.node', async () => {
                (0, vitest_1.it)('returns the node under the cursor on simple cases', async () => {
                    const context = '{{ "hey" }}\n\n{{ product.id }}{% ren█ %}';
                    const { completionContext } = createLiquidParamsFromContext(context);
                    (0, vitest_1.expect)(completionContext).to.exist;
                    const { node } = completionContext;
                    expectPath(node, 'name').to.eql('ren');
                    expectPath(node, 'markup').to.eql('');
                    expectPath(node, 'type').to.eql('LiquidTag');
                    expectPath(node, 'position.start').to.eql(29);
                    expectPath(node, 'position.end').to.eql(37);
                });
                (0, vitest_1.it)('returns the node under the cursor on nested contexts', async () => {
                    const context = `
            {% if product.compare_at_price > product.price %}
              <div>
                <h1>The product {{ product.tit█ }} is on sale!</h1>
              </div>
            {% endif %}
          `;
                    const { completionContext } = createLiquidParamsFromContext(context);
                    (0, vitest_1.expect)(completionContext).to.exist;
                    const { node } = completionContext;
                    expectPath(node, 'type').to.eql('VariableLookup');
                    expectPath(node, 'name').to.eql('product');
                    expectPath(node, 'lookups.0.type').to.eql('String');
                    expectPath(node, 'lookups.0.position.start').to.eql(126);
                    expectPath(node, 'lookups.0.position.end').to.eql(129);
                    expectPath(node, 'lookups.0.value').to.eql('tit');
                    expectPath(node, 'position.start').to.eql(118);
                    expectPath(node, 'position.end').to.eql(129);
                });
            });
            (0, vitest_1.it)("returns undefined when you're outside the node", async () => {
                const contexts = [
                    `{% assign x = b %}█`,
                    `<a href="...">█`,
                    `<a href="...">█`,
                    `{% for x in y rev %}█`,
                    `{% cycle %}█`,
                    `{% if %}█`,
                    `{% assign x %}`,
                    `<img>█`,
                    `<self-closing />█`,
                    `<a></a>█`,
                ];
                for (const context of contexts) {
                    const { completionContext } = createLiquidParamsFromContext(context);
                    const { node } = completionContext;
                    (0, vitest_1.expect)(node, context).to.eql(undefined);
                }
            });
            (0, vitest_1.it)('returns the HtmlVoidElement node when inside the tag', async () => {
                const context = '<img█';
                const { completionContext } = createLiquidParamsFromContext(context);
                const { node } = completionContext;
                expectPath(node, 'type', context).to.eql('HtmlVoidElement');
            });
            (0, vitest_1.it)("returns the TextNode when you're completing an HTML tag name", async () => {
                const contexts = [
                    `<█`,
                    `<a█ href="...">`,
                    `</█`,
                    `</a█ href="...">`,
                    `<h1></h█ href="...">`,
                ];
                for (const context of contexts) {
                    const { completionContext } = createLiquidParamsFromContext(context);
                    const { node } = completionContext;
                    expectPath(node, 'type', context).to.eql('TextNode');
                }
            });
            (0, vitest_1.it)('returns a tag', async () => {
                const contexts = [
                    `{% t█ %}`,
                    `{% end█ %}`,
                    `{% if cond %}{% end█ %}`,
                    `{% for markup as string █ %}`,
                    `{% if markup as string █ %}`,
                    `{% for x in y reversed █ %}`,
                    `{% for x in y reversed limit: 10 █ %}`,
                ];
                for (const context of contexts) {
                    const { completionContext } = createLiquidParamsFromContext(context);
                    const { node } = completionContext;
                    expectPath(node, 'type', context).to.eql('LiquidTag');
                }
            });
            (0, vitest_1.it)('returns a variable lookup', async () => {
                const contexts = [
                    `{{ a█`,
                    `{{ a.b█`,
                    `{{ a['b█`,
                    `{{ a.b.c█`,
                    `{% echo a█ %}`,
                    `{% echo a.b█ %}`,
                    `{% echo a['b█ %}`,
                    `{% assign x = a█ %}`,
                    `{% assign x = a.b█ %}`,
                    `{% assign x = a['b█ %}`,
                    `{% for a in b█ %}`,
                    `{% for a in b reversed limit: a█ %}`,
                    `{% paginate b by a█ %}`,
                    `{% paginate b by col, window_size: a█ %}`,
                    `{% if a█ %}`,
                    `{% if a > b█ %}`,
                    `{% if a > b or c█ %}`,
                    `{% if a > b or c > d█ %}`,
                    `{% elsif a > b█ %}`,
                    `{% when a█ %}`,
                    `{% when a, b█ %}`,
                    `{% cycle a█ %}`,
                    `{% cycle 'foo', a█ %}`,
                    `{% cycle 'foo': a█ %}`,
                    `{% render 'snip', var: a█ %}`,
                    `{% render 'snip' for col█ as item %}`,
                    `{% render 'snip' with object█ as name %}`,
                    `{% liquid
            echo a█
           %}`,
                    `{% for x in (1..a█) %}`,
                    // `{% paginate a█ by 50 %}`,
                    `<a-{{ a█ }}`,
                    `<a data-{{ a█ }}`,
                    `<a data={{ a█ }}`,
                    `<a data="{{ a█ }}"`,
                    `<a data='x{{ a█ }}'`,
                ];
                for (const context of contexts) {
                    const { completionContext } = createLiquidParamsFromContext(context);
                    const { node } = completionContext;
                    expectPath(node, 'type', context).to.eql('VariableLookup');
                }
            });
            (0, vitest_1.it)('returns a variable lookup (placeholder mode)', async () => {
                const contexts = [
                    `{{ █`,
                    `{{ a.█`,
                    `{{ a['█`,
                    `{{ a.b.█`,
                    `{% echo █ %}`,
                    `{% echo a.█ %}`,
                    `{% echo a['█ %}`,
                    `{% assign x = █ %}`,
                    `{% assign x = a.█ %}`,
                    `{% assign x = a['█ %}`,
                    `{% for a in █ %}`,
                    `{% for a in b reversed limit: █ %}`,
                    `{% paginate b by █ %}`,
                    `{% paginate b by col, window_size: █ %}`,
                    `{% if █ %}`,
                    `{% if a > █ %}`,
                    `{% if a > b or █ %}`,
                    `{% if a > b or c > █ %}`,
                    `{% elsif a > █ %}`,
                    `{% when █ %}`,
                    `{% when a, █ %}`,
                    `{% cycle █ %}`,
                    `{% cycle 'foo', █ %}`,
                    `{% cycle 'foo': █ %}`,
                    `{% render 'snip', var: █ %}`,
                    `{% render 'snip' for █ as item %}`,
                    `{% render 'snip' with █ as name %}`,
                    `{% for x in (1..█) %}`,
                    // `{% paginate a█ by 50 %}`,
                    `<a-{{ █ }}`,
                    `<a data-{{ █ }}`,
                    `<a data={{ █ }}`,
                    `<a data="{{ █ }}"`,
                    `<a data='x{{ █ }}'`,
                ];
                for (const context of contexts) {
                    const { completionContext } = createLiquidParamsFromContext(context);
                    (0, vitest_1.expect)(completionContext).to.exist;
                    const { node } = completionContext;
                    expectPath(node, 'type', context).to.eql('VariableLookup');
                }
            });
        });
    });
});
function expectPath(ast, path, message) {
    if (!ast)
        return (0, vitest_1.expect)(ast, message);
    return (0, vitest_1.expect)(deepGet(path.split('.'), ast), message);
}
function deepGet(path, obj) {
    return path.reduce((curr, k) => {
        if (curr && curr[k] !== undefined)
            return curr[k];
        return undefined;
    }, obj);
}
function createLiquidParamsFromContext(context, cursorPosition = calculatePosition(context)) {
    const regex = new RegExp('█', 'g');
    const documentManager = new documents_1.DocumentManager();
    const uri = 'file:///path/to/file.liquid';
    documentManager.open(uri, context.replace(regex, ''), 1);
    const params = mockCompletionParams({ position: cursorPosition });
    return (0, LiquidCompletionParams_1.createLiquidCompletionParams)(documentManager.get(uri), params);
}
function calculatePosition(context) {
    const index = context.indexOf('█');
    const lines = context.substring(0, index === -1 ? context.length : index).split('\n');
    const line = lines.length - 1;
    const character = lines[line].length;
    return { line, character };
}
function mockCompletionParams(params = {}) {
    return {
        position: {
            character: 0,
            line: 0,
        },
        textDocument: {
            uri: '/path/to/file.liquid',
        },
        ...params,
    };
}
//# sourceMappingURL=LiquidCompletionParams.spec.js.map