"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const theme_check_common_1 = require("@shopify/theme-check-common");
const documents_1 = require("../documents");
const DiagnosticsManager_1 = require("./DiagnosticsManager");
const runChecks_1 = require("./runChecks");
const LiquidFilter = {
    meta: {
        code: 'LiquidFilter',
        name: 'Complains about every LiquidFilter',
        docs: {
            description: 'Complains about every LiquidFilter',
            recommended: true,
        },
        type: theme_check_common_1.SourceCodeType.LiquidHtml,
        severity: theme_check_common_1.Severity.ERROR,
        schema: {},
        targets: [],
    },
    create(context) {
        return {
            LiquidFilter: async (node) => {
                context.report({
                    message: 'Liquid filter can not be used',
                    startIndex: node.position.start,
                    endIndex: node.position.end,
                });
            },
        };
    },
};
(0, vitest_1.describe)('Module: runChecks', () => {
    let diagnosticsManager;
    let documentManager;
    let connection;
    let runChecks;
    (0, vitest_1.beforeEach)(() => {
        connection = {
            sendDiagnostics: vitest_1.vi.fn(),
        };
        documentManager = new documents_1.DocumentManager();
        diagnosticsManager = new DiagnosticsManager_1.DiagnosticsManager(connection);
        runChecks = (0, runChecks_1.makeRunChecks)(documentManager, diagnosticsManager, {
            findRootURI: async () => 'browser:///',
            fileExists: async () => true,
            fileSize: async () => 420,
            getDefaultTranslationsFactory: () => async () => ({}),
            getDefaultLocaleFactory: () => async () => 'en',
            loadConfig: async () => ({
                settings: {},
                checks: [LiquidFilter],
                root: '/',
            }),
            themeDocset: {
                filters: async () => [],
                objects: async () => [],
                tags: async () => [],
            },
            schemaValidators: {
                validateSectionSchema: async () => ({}),
            },
        });
    });
    (0, vitest_1.it)('should send diagnostics when there are errors', async () => {
        const fileURI = 'browser:///input.liquid';
        const fileContents = `{{ 'any' | filter }}`;
        const fileVersion = 0;
        documentManager.open(fileURI, fileContents, fileVersion);
        await runChecks(['browser:///input.liquid']);
        (0, vitest_1.expect)(connection.sendDiagnostics).toBeCalled();
        (0, vitest_1.expect)(connection.sendDiagnostics).toBeCalledWith({
            uri: fileURI,
            version: fileVersion,
            diagnostics: [
                {
                    source: 'theme-check',
                    code: 'LiquidFilter',
                    message: 'Liquid filter can not be used',
                    severity: 1,
                    range: {
                        start: {
                            line: 0,
                            character: 8,
                        },
                        end: {
                            line: 0,
                            character: 17,
                        },
                    },
                },
            ],
        });
    });
    (0, vitest_1.it)('should send an empty array when the errors were cleared', async () => {
        const fileURI = 'browser:///input.liquid';
        const fileContentsWithError = `{{ 'any' | filter }}`;
        const fileContentsWithoutError = `{{ 'any' }}`;
        let fileVersion = 1;
        // Open and have errors
        documentManager.open(fileURI, fileContentsWithError, fileVersion);
        await runChecks(['browser:///input.liquid']);
        // Change doc to fix errors
        fileVersion = 2;
        documentManager.change(fileURI, fileContentsWithoutError, fileVersion);
        await runChecks(['browser:///input.liquid']);
        (0, vitest_1.expect)(connection.sendDiagnostics).toBeCalledTimes(2);
        (0, vitest_1.expect)(connection.sendDiagnostics).toHaveBeenLastCalledWith({
            uri: fileURI,
            version: fileVersion,
            diagnostics: [],
        });
    });
    (0, vitest_1.it)('should send diagnostics per URI when there are errors', async () => {
        const files = [
            {
                fileURI: 'browser:///input1.liquid',
                fileContents: `{{ 'any' | filter }}`,
                fileVersion: 0,
                diagnostics: [
                    {
                        source: 'theme-check',
                        code: 'LiquidFilter',
                        message: 'Liquid filter can not be used',
                        severity: 1,
                        range: {
                            start: {
                                line: 0,
                                character: 8,
                            },
                            end: {
                                line: 0,
                                character: 17,
                            },
                        },
                    },
                ],
            },
            {
                fileURI: 'browser:///input2.liquid',
                // same but on a new line
                fileContents: `\n{{ 'any' | filter }}`,
                fileVersion: 0,
                diagnostics: [
                    {
                        source: 'theme-check',
                        code: 'LiquidFilter',
                        message: 'Liquid filter can not be used',
                        severity: 1,
                        range: {
                            start: {
                                line: 1,
                                character: 8,
                            },
                            end: {
                                line: 1,
                                character: 17,
                            },
                        },
                    },
                ],
            },
        ];
        files.forEach(({ fileURI, fileContents, fileVersion }) => {
            documentManager.open(fileURI, fileContents, fileVersion);
        });
        await runChecks(['browser:///input1.liquid']);
        files.forEach(({ fileURI, fileVersion, diagnostics }) => {
            (0, vitest_1.expect)(connection.sendDiagnostics).toBeCalledWith({
                uri: fileURI,
                version: fileVersion,
                diagnostics,
            });
        });
    });
    (0, vitest_1.it)('should use the contents of the default translations file buffer (if any) instead of the result of the factory', async () => {
        const defaultURI = 'browser:///locales/en.default.json';
        const frURI = 'browser:///locales/fr.json';
        const files = {
            [defaultURI]: JSON.stringify({ hello: 'hello' }),
            [frURI]: JSON.stringify({ hello: 'bonjour', hi: 'salut' }),
        };
        const matchingTranslation = theme_check_common_1.allChecks.filter((c) => c.meta.code === 'MatchingTranslations');
        (0, vitest_1.expect)(matchingTranslation).to.have.lengthOf(1);
        runChecks = (0, runChecks_1.makeRunChecks)(documentManager, diagnosticsManager, {
            findRootURI: async () => 'browser:///',
            fileExists: async () => true,
            fileSize: async () => 420,
            getDefaultTranslationsFactory: () => async () => JSON.parse(files[defaultURI]),
            getDefaultLocaleFactory: () => async () => 'en',
            loadConfig: async () => ({
                settings: {},
                checks: matchingTranslation,
                root: '/',
            }),
            themeDocset: {
                filters: async () => [],
                objects: async () => [],
                tags: async () => [],
            },
            schemaValidators: {
                validateSectionSchema: async () => ({}),
            },
        });
        // Open and have errors
        documentManager.open(frURI, files[frURI], 0);
        await runChecks([frURI]);
        (0, vitest_1.expect)(connection.sendDiagnostics).toHaveBeenCalledWith({
            uri: frURI,
            version: 0,
            diagnostics: [
                {
                    source: 'theme-check',
                    code: 'MatchingTranslations',
                    codeDescription: { href: vitest_1.expect.any(String) },
                    message: `A default translation for 'hi' does not exist`,
                    severity: 1,
                    range: {
                        end: {
                            character: 31,
                            line: 0,
                        },
                        start: {
                            character: 19,
                            line: 0,
                        },
                    },
                },
            ],
        });
        // Change the contents of the defaultURI buffer, expect frURI to be fixed
        documentManager.open(defaultURI, files[defaultURI], 0);
        documentManager.change(defaultURI, JSON.stringify({ hello: 'hello', hi: 'hi' }), 1);
        connection.sendDiagnostics.mockClear();
        await runChecks([frURI]);
        (0, vitest_1.expect)(connection.sendDiagnostics).toHaveBeenCalledWith({
            uri: frURI,
            version: 0,
            diagnostics: [],
        });
    });
});
//# sourceMappingURL=runChecks.spec.js.map